@@ -31,7 +31,7 @@
 
 Base::Base(MProcess &mproc, PartMan &pman, Ui::MeInstall &ui,
     const QSettings &appConf, const QCommandLineParser &appArgs)
-    : QObject(ui.boxMain), proc(mproc), gui(ui), partman(pman)
+    : proc(mproc), gui(ui), partman(pman)
 {
     mediacheck = appArgs.isSet("media-check");
     if (!mediacheck) nomediacheck = appArgs.isSet("no-media-check");
@@ -43,7 +43,9 @@ Base::Base(MProcess &mproc, PartMan &pman, Ui::MeInstall &ui,
 void Base::scanMedia()
 {
     QSettings liveInfo("/live/config/initrd.out", QSettings::IniFormat);
-    const QString &toramMP = liveInfo.value("TORAM_MP", "/live/to-ram").toString();
+    const QString &sqpath = '/' + liveInfo.value("SQFILE_PATH", "antiX").toString();
+    const QString &sqtoram = liveInfo.value("TORAM_MP", "/live/to-ram").toString() + sqpath;
+    const QString &sqloc = liveInfo.value("SQFILE_DIR", "/live/boot-dev/antiX").toString();
 
     // Check the installation media for errors (skip if not required).
     bool checkmd5 = false;
@@ -58,12 +60,11 @@ void Base::scanMedia()
             }
         }
     }
-    if(checkmd5) proc.log("No media check (checkmd5)", MProcess::Standard);
-    else if(nomediacheck) proc.log("No media check", MProcess::Standard);
+    if(checkmd5) proc.log("No media check (checkmd5)");
+    else if(nomediacheck) proc.log("No media check");
     else {
-        QString msrc = liveInfo.value("SQFILE_DIR", "/live/boot-dev/antiX").toString();
-        if (!QFile::exists(msrc)) msrc = toramMP + "/antiX";
-        checkMediaMD5(msrc);
+        const QString &sqfile = liveInfo.value("SQFILE_NAME", "linuxfs").toString();
+        checkMediaMD5(QFile::exists(sqtoram+'/'+sqfile) ? sqtoram : sqloc, sqfile);
     }
 
     bootSource = "/live/aufs/boot";
@@ -76,12 +77,12 @@ void Base::scanMedia()
     proc.exec("du", {"-scb", bootSource}, nullptr, true);
     partman.bootSpaceNeeded = proc.readOut(true).section('\n', -1).section('\t', 0, 0).toLongLong();
 
-    QString infile = liveInfo.value("SQFILE_FULL", "/live/boot-dev/antiX/linuxfs").toString() + ".info";
-    if (!QFile::exists(infile)) infile = toramMP + "/antiX/linuxfs.info";
     bool floatOK = false;
+    QString infile = sqtoram + "/linuxfs.info";
+    if (!QFile::exists(infile)) infile = sqloc + "/linuxfs.info";
     if (QFile::exists(infile)) {
         const QSettings squashInfo(infile, QSettings::IniFormat);
-        partman.rootSpaceNeeded = 1024 * squashInfo.value("UncompressedSizeKB").toFloat(&floatOK);
+        partman.rootSpaceNeeded = squashInfo.value("UncompressedSizeKB").toFloat(&floatOK) * KB;
     }
     if (!floatOK) {
         rootSources.prepend("-scb");
@@ -93,7 +94,7 @@ void Base::scanMedia()
     // Account for persistent root.
     if (QFileInfo::exists("/live/perist-root")) {
         proc.shell("df /live/persist-root --output=used --total |tail -n1", nullptr, true);
-        const long long rootfs_size = proc.readOut().toLongLong() * 1024;
+        const long long rootfs_size = proc.readOut().toLongLong() * KB;
         qDebug() << "rootfs size is " << rootfs_size;
         // probaby conservative, as rootfs will likely have some overlap with linuxfs.
         partman.rootSpaceNeeded += rootfs_size;
@@ -102,7 +103,7 @@ void Base::scanMedia()
     qDebug() << "Minimum space:" << partman.bootSpaceNeeded << "(boot)," << partman.rootSpaceNeeded << "(root)";
 }
 
-void Base::checkMediaMD5(const QString &path)
+void Base::checkMediaMD5(const QString &path, const QString &sqfs)
 {
     checking = true;
     const QString osplash = gui.labelSplash->text();
@@ -118,7 +119,7 @@ void Base::checkMediaMD5(const QString &path)
     static const char *failmsg = QT_TR_NOOP("The installation media is corrupt.");
     // Obtain a list of MD5 hashes and their files.
     QDirIterator it(path, {"*.md5"}, QDir::Files);
-    QStringList missing("linuxfs");
+    QStringList missing(sqfs);
     if (!QFile::exists("/live/config/did-toram") || QFile::exists("/live/config/toram-all")) {
         missing << "vmlinuz" << "initrd.gz";
     }
@@ -144,7 +145,7 @@ void Base::checkMediaMD5(const QString &path)
     std::unique_ptr<char[]> buf(new char[bufsize]);
     qint64 bprog = 0;
     for(const FileHash &fh : hashes) {
-        QListWidgetItem *logEntry = proc.log("Check MD5: " + fh.path, MProcess::Standard);
+        QListWidgetItem *logEntry = proc.log("Check MD5: " + fh.path);
         QFile file(fh.path);
         if (!file.open(QFile::ReadOnly)) throw failmsg;
         QCryptographicHash hash(QCryptographicHash::Md5);
@@ -161,7 +162,7 @@ void Base::checkMediaMD5(const QString &path)
         proc.log(logEntry);
     }
     gui.labelSplash->setText(osplash);
-    if (!checking) proc.log("Check halted", MProcess::Standard);
+    if (!checking) proc.log("Check halted");
     checking = false;
 }
 
@@ -179,7 +180,7 @@ void Base::haltCheck(bool silent)
 
 void Base::install()
 {
-    proc.log(__PRETTY_FUNCTION__);
+    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
     if (proc.halted()) return;
     proc.advance(1, 2);
 
@@ -215,11 +216,6 @@ void Base::install()
     qDebug() << "Desktop menu";
     proc.exec("chroot", {"/mnt/antiX", "desktop-menu", "--write-out-global"});
 
-    // if POPULATE_MEDIA_MOUNTPOINTS is true in gazelle-installer-data, then use the --mntpnt switch
-    partman.makeFstab(populateMediaMounts);
-    if (!partman.fixCryptoSetup()) {
-        throw QT_TR_NOOP("Failed to finalize encryption setup.");
-    }
     // Disable hibernation inside initramfs.
     if (partman.isEncrypt("SWAP")) {
         proc.exec("touch", {"/mnt/antiX/initramfs-tools/conf.d/resume"});
@@ -235,16 +231,6 @@ void Base::install()
     if (!QFileInfo("/live/linux/home/demo").isDir())
         proc.exec("/bin/rm", {"-rf", "/mnt/antiX/home/demo"});
 
-    // if POPULATE_MEDIA_MOUNTPOINTS is true in gazelle-installer-data, don't clean /media folder
-    // modification to preserve points that are still mounted.
-    if (!populateMediaMounts) {
-        proc.shell("/bin/rmdir --ignore-fail-on-non-empty /mnt/antiX/media/sd*");
-    }
-
-    // guess localtime vs UTC
-    proc.shell("guess-hwclock", nullptr, true);
-    if (proc.readOut() == "localtime") gui.checkLocalClock->setChecked(true);
-
     // create a /etc/machine-id file and /var/lib/dbus/machine-id file
     proc.exec("/bin/mount", {"--rbind", "--make-rslave", "/dev", "/mnt/antiX/dev"});
     proc.setChRoot("/mnt/antiX");
@@ -262,11 +248,20 @@ void Base::install()
         proc.shell("/bin/mv -f /mnt/antiX/etc/rc2.d/S*virtualbox-guest-utils /mnt/antiX/etc/rc2.d/K01virtualbox-guest-utils >/dev/null 2>&1");
         proc.shell("/bin/mv -f /mnt/antiX/etc/rcS.d/S*virtualbox-guest-x11 /mnt/antiX/etc/rcS.d/K21virtualbox-guest-x11 >/dev/null 2>&1");
     }
+
+    partman.installTabs();
+    // if POPULATE_MEDIA_MOUNTPOINTS is true in gazelle-installer-data, then use the --mntpnt switch
+    if (populateMediaMounts) {
+        proc.shell("/sbin/make-fstab -O --install=/mnt/antiX --mntpnt=/media");
+    } else {
+        // Otherwise, clean /media folder - modification to preserve points that are still mounted.
+        proc.shell("/bin/rmdir --ignore-fail-on-non-empty /mnt/antiX/media/sd*");
+    }
 }
 
 void Base::copyLinux()
 {
-    proc.log(__PRETTY_FUNCTION__);
+    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
     if (proc.halted()) return;
 
     // copy most except usr, mnt and home
@@ -296,8 +291,8 @@ void Base::copyLinux()
     QListWidgetItem *logEntry = proc.log(joined, MProcess::Exec);
 
     QEventLoop eloop;
-    connect(&proc, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), &eloop, &QEventLoop::quit);
-    connect(&proc, &QProcess::readyRead, &eloop, &QEventLoop::quit);
+    QObject::connect(&proc, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), &eloop, &QEventLoop::quit);
+    QObject::connect(&proc, &QProcess::readyRead, &eloop, &QEventLoop::quit);
     proc.start(prog, args);
     long ncopy = 0;
     while (proc.state() != QProcess::NotRunning) {
@@ -305,8 +300,8 @@ void Base::copyLinux()
         ncopy += proc.readAllStandardOutput().count('\n');
         proc.status(ncopy);
     }
-    disconnect(&proc, &QProcess::readyRead, nullptr, nullptr);
-    disconnect(&proc, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), nullptr, nullptr);
+    QObject::disconnect(&proc, &QProcess::readyRead, nullptr, nullptr);
+    QObject::disconnect(&proc, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), nullptr, nullptr);
 
     const QByteArray &StdErr = proc.readAllStandardError();
     if (!StdErr.isEmpty()) {
