@@ -1,8 +1,7 @@
 /***************************************************************************
  * Basic partition manager for the installer.
- ***************************************************************************
  *
- *   Copyright (C) 2019, 2020-2022 by AK-47
+ *   Copyright (C) 2019-2023 by AK-47
  *   Transplanted code, marked with comments further down this file:
  *    - Copyright (C) 2003-2010 by Warren Woodford
  *    - Heavily edited, with permision, by anticapitalista for antiX 2011-2014.
@@ -22,11 +21,11 @@
  * This file is part of the gazelle-installer.
  ***************************************************************************/
 
-#include <sys/sysinfo.h>
 #include <sys/stat.h>
 #include <QDebug>
 #include <QLocale>
 #include <QFileInfo>
+#include <QDir>
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QJsonArray>
@@ -41,11 +40,9 @@
 #include <QDialogButtonBox>
 
 #include "msettings.h"
+#include "autopart.h"
 #include "partman.h"
 
-#define PARTMAN_SAFETY_MB 8 // 1MB at start + Compensate for rounding errors.
-#define PARTMAN_MAX_PARTS 128 // Maximum number of partitions Linux supports.
-
 PartMan::PartMan(MProcess &mproc, Ui::MeInstall &ui, const QSettings &appConf, const QCommandLineParser &appArgs)
     : QAbstractItemModel(ui.boxMain), proc(mproc), root(DeviceItem::Unknown), gui(ui)
 {
@@ -152,6 +149,7 @@ void PartMan::scan(DeviceItem *drvstart)
             partit->flags.oldLayout = true;
             partit->device = jsonPart["name"].toString();
             partit->path = jsonPart["path"].toString();
+            partit->uuid = jsonPart["uuid"].toString();
             partit->order = order.indexOf(partit->device);
             partit->size = jsonPart["size"].toVariant().toLongLong();
             partit->physec = jsonPart["phy-sec"].toInt();
@@ -163,8 +161,9 @@ void PartMan::scan(DeviceItem *drvstart)
             if ((partflags & 0x80) || (partflags & 0x04)) partit->setActive(true);
             partit->mapCount = jsonPart["children"].toArray().count();
             partit->flags.curESP = partTypeName.startsWith("EFI "); // "System"/"(FAT-12/16/32)"
-            partit->flags.bootRoot = (!bootUUID.isEmpty() && jsonPart["uuid"] == bootUUID);
+            partit->flags.bootRoot = (!bootUUID.isEmpty() && partit->uuid == bootUUID);
             partit->curFormat = jsonPart["fstype"].toString();
+            if (partTypeName == "BIOS boot") partit->curFormat = "BIOS-GRUB";
             // Touching MacOS first drive, or MS LDM may brick the system.
             if ((proc.detectMac() && partit->device.startsWith("sda"))
                 || partTypeName.startsWith("Microsoft LDM")) {
@@ -253,11 +252,12 @@ void PartMan::scanVirtualDevices(bool rescan)
             devit = new DeviceItem(DeviceItem::VirtualBD, vdlit);
             devit->device = jsonDev["name"].toString();
             devit->path = path;
+            devit->uuid = jsonDev["uuid"].toString();
             devit->flags.rotational = rota;
             devit->discgran = discgran;
             devit->size = size;
             devit->physec = physec;
-            devit->flags.bootRoot = (!bootUUID.isEmpty() && jsonDev["uuid"] == bootUUID);
+            devit->flags.bootRoot = (!bootUUID.isEmpty() && devit->uuid == bootUUID);
             devit->curLabel = label;
             devit->curFormat = jsonDev["fstype"].toString();
             devit->flags.volCrypto = crypto;
@@ -304,7 +304,7 @@ bool PartMan::manageConfig(MSettings &config, bool save)
             if (partCount > PARTMAN_MAX_PARTS) return false;
         }
         // Partition configuration.
-        const long long sizeMax = drvit->size - (PARTMAN_SAFETY_MB * 1048576);
+        const long long sizeMax = drvit->size - PARTMAN_SAFETY;
         long long sizeTotal = 0;
         for (int ixPart = 0; ixPart < partCount; ++ixPart) {
             DeviceItem *partit = nullptr;
@@ -428,7 +428,7 @@ void PartMan::treeSelChange()
         if (!islocked && isold && isdrive) gui.pushPartAdd->setEnabled(false);
         else if (!isold) {
             gui.pushPartAdd->setEnabled(drvit->childCount() < PARTMAN_MAX_PARTS
-                && twit->driveFreeSpace(true) > 1048576);
+                && twit->driveFreeSpace(true) > 1*MB);
         }
     } else {
         gui.pushPartClear->setEnabled(false);
@@ -503,25 +503,23 @@ void PartMan::treeMenu(const QPoint &)
     } else if (twit->type == DeviceItem::Drive) {
         QAction *actAdd = menu.addAction(tr("&Add partition"));
         actAdd->setEnabled(gui.pushPartAdd->isEnabled());
+        QAction *actBuild = menu.addAction(tr("&Build layout..."));
         menu.addSeparator();
         QAction *actClear = menu.addAction(tr("New &layout"));
         QAction *actReset = menu.addAction(tr("&Reset layout"));
-        QMenu *menuTemplates = menu.addMenu(tr("&Templates"));
-        const QAction *actBasic = menuTemplates->addAction(tr("&Standard install"));
-        QAction *actCrypto = menuTemplates->addAction(tr("&Encrypted system"));
 
         const bool locked = twit->isLocked();
         actClear->setDisabled(locked);
         actReset->setDisabled(locked);
-        menuTemplates->setDisabled(locked);
-        actCrypto->setVisible(gui.boxCryptoPass->isVisible());
+        actBuild->setDisabled(locked);
 
         QAction *action = menu.exec(QCursor::pos());
         if (action == actAdd) partAddClick(true);
         else if (action == actClear) partClearClick(true);
-        else if (action == actBasic) twit->layoutDefault(-1, false);
-        else if (action == actCrypto) twit->layoutDefault(-1, true);
-        else if (action == actReset) {
+        else if (action == actBuild) {
+            if (autopart) autopart->builderGUI(twit);
+            treeSelChange();
+        } else if (action == actReset) {
             gui.boxMain->setEnabled(false);
             scan(twit);
             gui.boxMain->setEnabled(true);
@@ -565,8 +563,9 @@ void PartMan::partRemoveClick(bool)
     if (!devit) return;
     DeviceItem *drvit = devit->parent();
     if (!drvit) return;
+    const bool notSub = (devit->type != DeviceItem::Subvolume);
     delete devit;
-    if (devit->type != DeviceItem::Subvolume) {
+    if (notSub) {
         drvit->labelParts();
         treeSelChange();
     }
@@ -793,8 +792,11 @@ bool PartMan::composeValidate(bool automatic, const QString &project)
                 const QString &use = partit->realUseFor();
                 QString actmsg;
                 if (drvit->flags.oldLayout) {
-                    if (use.isEmpty()) continue;
-                    else if (partit->willFormat()) actmsg = tr("Format %1 to use for %2");
+                    if (use.isEmpty()) {
+                        if (partit->curFormat == "BIOS-GRUB") hasBiosGrub = true;
+                        continue;
+                    }
+                    if (partit->willFormat()) actmsg = tr("Format %1 to use for %2");
                     else if (use != "/") actmsg = tr("Reuse (no reformat) %1 as %2");
                     else actmsg = tr("Delete the data on %1 except for /home, to use for %2");
                 } else {
@@ -950,10 +952,37 @@ int PartMan::countPrepSteps()
     }
     return nstep;
 }
+void PartMan::prepStorage()
+{
+    preparePartitions();
+    formatPartitions();
+    mountPartitions();
+    // Run blkid -c /dev/null to freshen UUID cache
+    proc.exec("blkid", {"-c", "/dev/null"});
+    // Refresh the UUIDs
+    proc.exec("lsblk", {"--list", "-bJo", "PATH,UUID"}, nullptr, true);
+    const QJsonObject &jsonObjBD = QJsonDocument::fromJson(proc.readOut(true).toUtf8()).object();
+    const QJsonArray &jsonBD = jsonObjBD["blockdevices"].toArray();
+    QMap<QString, DeviceItem *> alldevs;
+    for (DeviceItemIterator it(*this); DeviceItem *item = *it; it.next()) {
+        alldevs.insert(item->path, item);
+    }
+    for (const QJsonValue &jsonDev : jsonBD) {
+        DeviceItem *devit = alldevs.value(jsonDev["path"].toString());
+        if (devit) devit->uuid = jsonDev["uuid"].toString();
+    }
+}
+void PartMan::installTabs()
+{
+    makeFstab();
+    if (!fixCryptoSetup()) {
+        throw QT_TR_NOOP("Failed to finalize encryption setup.");
+    }
+}
 
 void PartMan::preparePartitions()
 {
-    proc.log(__PRETTY_FUNCTION__);
+    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
     static const char *const msgfail = QT_TR_NOOP("Failed to prepare required partitions.");
 
     // Detach all existing partitions.
@@ -977,7 +1006,7 @@ void PartMan::preparePartitions()
     proc.exec("swapon", {"--show=NAME", "--noheadings"}, nullptr, true);
     const QStringList swaps = proc.readOutLines();
     for (const QString &devpath : qAsConst(listToUnmount)) {
-        if (swaps.contains(devpath)) proc.exec("swapoff", {devpath});
+        if (swaps.contains(devpath)) proc.shell("grep -q " + devpath + " /proc/swaps && swapoff " + devpath);
         else proc.exec("/usr/bin/umount", {"-q", devpath});
     }
 
@@ -1028,7 +1057,7 @@ void PartMan::preparePartitions()
             long long start = 1; // start with 1 MB to aid alignment
             for (int ixdev = 0; ixdev<devCount; ++ixdev) {
                 DeviceItem *twit = drvit->child(ixdev);
-                const long long end = start + twit->size / 1048576;
+                const long long end = start + twit->size / MB;
                 const bool rc = proc.exec("parted", {"-s", "--align", "optimal", drvit->path,
                     "mkpart" , "primary", QString::number(start) + "MiB", QString::number(end) + "MiB"});
                 if (!rc) throw msgfail;
@@ -1055,7 +1084,7 @@ void PartMan::preparePartitions()
 
 void PartMan::formatPartitions()
 {
-    proc.log(__PRETTY_FUNCTION__);
+    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
 
     const QByteArray &encPass = (gui.radioEntireDisk->isChecked()
         ? gui.textCryptoPass : gui.textCryptoPassCust)->text().toUtf8();
@@ -1165,8 +1194,7 @@ void PartMan::loadKeyMaterial(const QString &keyfile)
     key.load(keyfile.toUtf8().constData(), -1);
     const int keylen = key.length();
     if (keylen > 0) {
-        proc.log(QStringLiteral("Loaded %1-byte key material: ").arg(keylen)
-            + keyfile, MProcess::Standard);
+        proc.log(QStringLiteral("Loaded %1-byte key material: ").arg(keylen) + keyfile);
     }
 }
 
@@ -1231,8 +1259,7 @@ bool PartMan::fixCryptoSetup()
     QTextStream out(&file);
     // Add devices to crypttab.
     for (auto &it : cryptAdd.toStdMap()) {
-        proc.exec("blkid", {"-s", "UUID", "-o", "value", "/dev/" + it.first}, nullptr, true);
-        out << it.second->devMapper << " /dev/disk/by-uuid/" << proc.readOut();
+        out << it.second->devMapper << " /dev/disk/by-uuid/" << it.second->uuid;
         if (noKey || it.first == keyDev) out << " none";
         else {
             if (isNewKey) {
@@ -1258,7 +1285,7 @@ bool PartMan::fixCryptoSetup()
 }
 
 // create /etc/fstab file
-bool PartMan::makeFstab(bool populateMediaMounts)
+bool PartMan::makeFstab()
 {
     QFile file("/mnt/antiX/etc/fstab");
     if (!file.open(QIODevice::WriteOnly)) return false;
@@ -1271,10 +1298,7 @@ bool PartMan::makeFstab(bool populateMediaMounts)
         qDebug() << "Creating fstab entry for:" << it.first << dev;
         // Device ID or UUID
         if (twit->willMap()) out << dev;
-        else {
-            proc.exec("blkid", {"-o", "value", "UUID", "-s", "UUID", dev}, nullptr, true);
-            out << "UUID=" + proc.readOut();
-        }
+        else out << "UUID=" << twit->uuid;
         // Mount point, file system
         proc.exec("blkid", {dev, "-o", "value", "-s", "TYPE"}, nullptr, true);
         const QString &fsfmt = proc.readOut();
@@ -1299,21 +1323,17 @@ bool PartMan::makeFstab(bool populateMediaMounts)
     }
     // EFI System Partition
     if (gui.radioBootESP->isChecked()) {
-        const QString &dev = "/dev/" + gui.comboBoot->currentData().toString();
-        proc.exec("blkid", {"-o", "value", "UUID", "-s", "UUID", dev}, nullptr, true);
-        out << "UUID=" << proc.readOut() << " /boot/efi vfat noatime,dmask=0002,fmask=0113 0 0\n";
+        DeviceItem *devit = findByPath("/dev/" + gui.comboBoot->currentData().toString());
+        if (!devit) return false;
+        out << "UUID=" << devit->uuid << " /boot/efi vfat noatime,dmask=0002,fmask=0113 0 0\n";
     }
     file.close();
-    if (populateMediaMounts) {
-        if (!proc.shell("/sbin/make-fstab -O --install=/mnt/antiX"
-            " --mntpnt=/media")) return false;
-    }
     return true;
 }
 
 void PartMan::mountPartitions()
 {
-    proc.log(__PRETTY_FUNCTION__);
+    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
     for (auto &it : mounts.toStdMap()) {
         if (it.first.at(0) != '/') continue;
         static const char *const msgfail = QT_TR_NOOP("Failed to mount partition.");
@@ -1349,7 +1369,7 @@ void PartMan::unmount()
         DeviceItem *twit = mit.value();
         if (!twit) continue;
         if (mit.key().startsWith("SWAP")) {
-            proc.exec("swapoff", {twit->mappedDevice()});
+            proc.shell("grep -q " + twit->mappedDevice() + " /proc/swaps &&  swapoff " + twit->mappedDevice());
         } else if (mit.key().at(0) == '/') {
             proc.exec("/bin/umount", {"-l", "/mnt/antiX" + mit.key()});
         }
@@ -1411,6 +1431,16 @@ DeviceItem *PartMan::findByPath(const QString &devpath) const
     }
     return nullptr;
 }
+DeviceItem *PartMan::findHostDev(const QString &path) const
+{
+    QString spath = path;
+    DeviceItem *devit = nullptr;
+    do {
+        spath = QDir(QFileInfo(spath).absolutePath()).path();
+        devit = mounts.value(spath);
+    } while(!devit && !spath.isEmpty() && spath != '/');
+    return devit;
+}
 
 /*************************\
  * Model View Controller *
@@ -1694,7 +1724,7 @@ void PartMan::notifyChange(class DeviceItem *item, int first, int last)
 DeviceItem::DeviceItem(enum DeviceType type, DeviceItem *parent, DeviceItem *preceding)
     : parentItem(parent), type(type)
 {
-    if (type == Partition) size = 1048576;
+    if (type == Partition) size = 1*MB;
     if (parent) {
         flags.rotational = parent->flags.rotational;
         discgran = parent->discgran;
@@ -1788,9 +1818,9 @@ QString DeviceItem::shownUseFor() const
 {
     const QString &use = realUseFor();
     if (use == "/") return "/ (root)";
-    else if (use == "ESP") return qApp->tr("EFI System Partition");
-    else if (use == "SWAP") return qApp->tr("swap space");
-    else if (use == "FORMAT") return qApp->tr("format only");
+    else if (use == "ESP") return tr("EFI System Partition");
+    else if (use == "SWAP") return tr("swap space");
+    else if (use == "FORMAT") return tr("format only");
     return use;
 }
 void DeviceItem::setActive(bool boot)
@@ -1815,11 +1845,11 @@ bool DeviceItem::isLocked() const
     }
     return (mapCount != 0);
 }
-inline bool DeviceItem::willUseGPT() const
+bool DeviceItem::willUseGPT() const
 {
     if (type != Drive) return false;
     if (flags.oldLayout) return flags.curGPT;
-    else if (size >= 2199023255552 || children.count() > 4) return true;
+    else if (size >= 2*TB || children.count() > 4) return true;
     else if (partman) return (partman->gptoverride || partman->proc.detectEFI());
     return false;
 }
@@ -1859,7 +1889,7 @@ QString DeviceItem::shownDevice() const
     if (type == Subvolume) return parentItem->device + '[' + label + ']';
     return device;
 }
-QStringList DeviceItem::allowedUsesFor(bool real) const
+QStringList DeviceItem::allowedUsesFor(bool real, bool all) const
 {
     if (!isVolume() && type != Subvolume) return QStringList();
     QStringList list;
@@ -1868,8 +1898,11 @@ QStringList DeviceItem::allowedUsesFor(bool real) const
     else {
         list.prepend("Format");
         if (type != VirtualBD) {
-            if (size <= 16777216) list << "BIOS-GRUB";
-            if (size <= 8589934592) list << "ESP" << "boot";
+            if (all || size <= 16*MB) list << "BIOS-GRUB";
+            if (all || size <= 8*GB) {
+                if (size < (2*TB - 512)) list << "ESP";
+                list << "boot";
+            }
         }
         list << "swap" << "home";
     }
@@ -1881,44 +1914,43 @@ QStringList DeviceItem::allowedUsesFor(bool real) const
 QStringList DeviceItem::allowedFormats() const
 {
     QStringList list;
-    bool allowPreserve = false, selPreserve = false;
+    const QString &use = realUseFor();
+    bool allowPreserve = false;
     if (type == Subvolume) list.append("CREATE");
     else if (isVolume()) {
-        const QString &use = realUseFor();
         if (use.isEmpty()) return QStringList();
         else if (use == "/boot") list.append("ext4");
         else if (use == "BIOS-GRUB") list.append("GRUB");
         else if (use == "ESP") {
             list.append("FAT32");
-            if (size <= 4294901760) list.append("FAT16");
-            if (size <= 33553920) list.append("FAT12");
-            selPreserve = allowPreserve = (list.contains(curFormat, Qt::CaseInsensitive)
+            if (size <= (4*GB - 64*KB)) list.append("FAT16");
+            if (size <= (32*MB - 512)) list.append("FAT12");
+            allowPreserve = (list.contains(curFormat, Qt::CaseInsensitive)
                 || !curFormat.compare("VFAT", Qt::CaseInsensitive));
         } else if (use == "SWAP") {
             list.append("SWAP");
-            selPreserve = allowPreserve = list.contains(curFormat, Qt::CaseInsensitive);
+            allowPreserve = list.contains(curFormat, Qt::CaseInsensitive);
         } else {
             list << "ext4" << "ext3" << "ext2";
             list << "f2fs" << "jfs" << "xfs" << "btrfs";
             if (use != "FORMAT") allowPreserve = list.contains(curFormat, Qt::CaseInsensitive);
-            if (use == "/home") selPreserve = allowPreserve;
         }
     }
     if (encrypt) allowPreserve = false;
     if (allowPreserve) {
-        if (selPreserve) list.prepend("PRESERVE");
+        if (use != "/") list.prepend("PRESERVE");
         else list.append("PRESERVE");
     }
     return list;
 }
 QString DeviceItem::shownFormat(const QString &fmt) const
 {
-    if (fmt == "CREATE") return qApp->tr("Create");
+    if (fmt == "CREATE") return tr("Create");
     else if (fmt != "PRESERVE") return fmt;
     else {
-        if (type == Subvolume) return qApp->tr("Preserve");
-        else if (realUseFor() != "/") return qApp->tr("Preserve (%1)").arg(curFormat);
-        else return qApp->tr("Preserve /home (%1)").arg(curFormat);
+        if (type == Subvolume) return tr("Preserve");
+        else if (realUseFor() != "/") return tr("Preserve (%1)").arg(curFormat);
+        else return tr("Preserve /home (%1)").arg(curFormat);
     }
 }
 bool DeviceItem::canMount() const
@@ -1930,7 +1962,7 @@ long long DeviceItem::driveFreeSpace(bool inclusive) const
 {
     const DeviceItem *drvit = parent();
     if (!drvit) drvit = this;
-    long long free = drvit->size - (PARTMAN_SAFETY_MB * 1048576);
+    long long free = drvit->size - PARTMAN_SAFETY;
     for (int ixi = drvit->children.count() - 1; ixi >= 0; --ixi) {
         DeviceItem *partit = drvit->children.at(ixi);
         if (inclusive || partit != this) free -= partit->size;
@@ -1938,12 +1970,11 @@ long long DeviceItem::driveFreeSpace(bool inclusive) const
     return free;
 }
 /* Convenience */
-DeviceItem *DeviceItem::addPart(int defaultMB, const QString &defaultUse, bool crypto)
+DeviceItem *DeviceItem::addPart(long long defaultSize, const QString &defaultUse, bool crypto)
 {
     DeviceItem *partit = new DeviceItem(DeviceItem::Partition, this);
     if (!defaultUse.isEmpty()) partit->usefor = defaultUse;
-    partit->size = defaultMB;
-    partit->size *= 1048576;
+    partit->size = defaultSize;
     partit->autoFill();
     if (partit->canEncrypt()) partit->encrypt = crypto;
     if (partman) partman->notifyChange(partit);
@@ -2040,59 +2071,6 @@ void DeviceItem::labelParts()
     if (partman) partman->resizeColumnsToFit();
 }
 
-int DeviceItem::layoutDefault(int rootPercent, bool crypto, bool updateTree)
-{
-    assert (partman != nullptr);
-    if (rootPercent<0) rootPercent = partman->gui.sliderPart->value();
-    if (updateTree) clear();
-    const long long driveSize = size / 1048576;
-    int rootFormatSize = static_cast<int>(driveSize - PARTMAN_SAFETY_MB);
-
-    // Boot partitions.
-    if (partman->proc.detectEFI()) {
-        if (updateTree) addPart(256, "ESP", crypto);
-        rootFormatSize -= 256;
-    } else if (driveSize >= 2097152 || partman->gptoverride) {
-        if (updateTree) addPart(1, "BIOS-GRUB", crypto);
-        rootFormatSize -= 1;
-    }
-    int rootMinMB = static_cast<int>(partman->rootSpaceNeeded / 1048576);
-    const int bootMinMB = static_cast<int>(partman->bootSpaceNeeded / 1048576);
-    if (!crypto) rootMinMB += bootMinMB;
-    else {
-        int bootFormatSize = 1024;
-        if (bootFormatSize < bootMinMB) bootFormatSize = static_cast<int>(partman->bootSpaceNeeded);
-        if (updateTree) addPart(bootFormatSize, "boot", crypto);
-        rootFormatSize -= bootFormatSize;
-    }
-    // Swap space.
-    int swapFormatSize = rootFormatSize-rootMinMB;
-    struct sysinfo sinfo;
-    if (sysinfo(&sinfo) != 0) sinfo.totalram = 2048;
-    else sinfo.totalram = (sinfo.totalram / (1048576 * 2)) * 3; // 1.5xRAM
-    sinfo.totalram /= 128; ++sinfo.totalram; sinfo.totalram *= 128; // Multiple of 128MB
-    if (swapFormatSize > static_cast<int>(sinfo.totalram)) swapFormatSize = static_cast<int>(sinfo.totalram);
-    int swapMaxMB = rootFormatSize / (20 * 128); ++swapMaxMB; swapMaxMB *= 128; // 5% root
-    if (swapMaxMB > 8192) swapMaxMB = 8192; // 8GB cap for the whole calculation.
-    if (swapFormatSize > swapMaxMB) swapFormatSize = swapMaxMB;
-    rootFormatSize -= swapFormatSize;
-    // Home
-    int homeFormatSize = rootFormatSize;
-    rootFormatSize = (rootFormatSize * rootPercent) / 100;
-    if (rootFormatSize < rootMinMB) rootFormatSize = rootMinMB;
-    homeFormatSize -= rootFormatSize;
-
-    if (updateTree) {
-        addPart(rootFormatSize, "root", crypto);
-        if (swapFormatSize>0) addPart(swapFormatSize, "swap", crypto);
-        if (homeFormatSize>0) addPart(homeFormatSize, "home", crypto);
-        labelParts();
-        driveAutoSetActive();
-        partman->treeSelChange();
-    }
-    return rootFormatSize;
-}
-
 // Return block device info that is suitable for a combo box.
 void DeviceItem::addToCombo(QComboBox *combo, bool warnNasty) const
 {
@@ -2205,7 +2183,7 @@ QSize DeviceItemDelegate::sizeHint(const QStyleOptionViewItem &option, const QMo
         }
         break;
     case PartMan::UseFor:
-        for (const QString &use : item->allowedUsesFor(false)) {
+        for (const QString &use : item->allowedUsesFor(false, false)) {
             const int uw = option.fontMetrics.boundingRect(use).width() + residue;
             if (uw > width) width = uw;
         }
@@ -2267,8 +2245,8 @@ void DeviceItemDelegate::setEditorData(QWidget *editor, const QModelIndex &index
     case PartMan::Size:
         {
             QSpinBox *spin = qobject_cast<QSpinBox *>(editor);
-            spin->setRange(0, static_cast<int>(item->driveFreeSpace() / 1048576));
-            spin->setValue(item->size / 1048576);
+            spin->setRange(0, static_cast<int>(item->driveFreeSpace() / MB));
+            spin->setValue(item->size / MB);
         }
         break;
     case PartMan::UseFor:
@@ -2276,7 +2254,7 @@ void DeviceItemDelegate::setEditorData(QWidget *editor, const QModelIndex &index
             QComboBox *combo = qobject_cast<QComboBox *>(editor);
             combo->clear();
             combo->addItem("");
-            combo->addItems(item->allowedUsesFor(false));
+            combo->addItems(item->allowedUsesFor(false, false));
             combo->setCurrentText(item->usefor);
         }
         break;
