@@ -1,12 +1,7 @@
 /***************************************************************************
  * Basic partition manager for the installer.
- ***************************************************************************
  *
- *   Copyright (C) 2019, 2020-2021 by AK-47
- *   Transplanted code, marked with comments further down this file:
- *    - Copyright (C) 2003-2010 by Warren Woodford
- *    - Heavily edited, with permision, by anticapitalista for antiX 2011-2014.
- *    - Heavily revised by dolphin oracle, adrian, and anticaptialista 2018.
+ *   Copyright (C) 2019-2023 by AK-47
  *   Licensed under the Apache License, Version 2.0 (the "License");
  *   you may not use this file except in compliance with the License.
  *   You may obtain a copy of the License at
@@ -37,8 +32,19 @@
 #include "msettings.h"
 #include "safecache.h"
 
+#define KB 1024LL
+#define MB 1048576LL
+#define GB 1073741824LL
+#define TB 1099511627776LL
+#define PB 1125899906842624LL
+#define EB 1152921504606846976LL
+
+#define PARTMAN_SAFETY (8*MB) // 1MB at start + Compensate for rounding errors.
+#define PARTMAN_MAX_PARTS 128 // Maximum number of partitions Linux supports.
+
 class DeviceItem
 {
+    Q_DECLARE_TR_FUNCTIONS(DeviceItem)
     friend class PartMan;
     friend class DeviceItemIterator;
     friend class DeviceItemDelegate;
@@ -46,10 +52,7 @@ class DeviceItem
     DeviceItem *parentItem = nullptr;
     class PartMan *partman = nullptr;
     int order = -1;
-    DeviceItem *addPart(int defaultMB, const QString &defaultUse, bool crypto);
-    void driveAutoSetActive();
     void autoFill(unsigned int changed = 0xFFFF);
-    void labelParts();
 public:
     DeviceItem *active = nullptr;
     DeviceItem *origin = nullptr;
@@ -72,7 +75,7 @@ public:
         bool curESP : 1;
         bool volCrypto : 1;
     } flags = {};
-    QString model, device, path, devMapper;
+    QString model, device, path, uuid, devMapper;
     QString label, curLabel;
     QString usefor;
     QString format, curFormat;
@@ -96,6 +99,10 @@ public:
     int indexOfChild(DeviceItem *child);
     int childCount() const;
     void sortChildren();
+    // Layout finishing
+    DeviceItem *addPart(long long defaultSize, const QString &defaultUse, bool crypto);
+    void driveAutoSetActive();
+    void labelParts();
     // Helpers
     static QString realUseFor(const QString &use);
     inline QString realUseFor() const { return realUseFor(usefor); }
@@ -110,14 +117,13 @@ public:
     QString mappedDevice() const;
     bool willMap() const;
     QString shownDevice() const;
-    QStringList allowedUsesFor(bool real = true) const;
+    QStringList allowedUsesFor(bool real = true, bool all = true) const;
     QStringList allowedFormats() const;
     QString shownFormat(const QString &fmt) const;
     inline bool isVolume() const { return (type == Partition || type == VirtualBD); }
     bool canMount() const;
     long long driveFreeSpace(bool inclusive = false) const;
     /* Convenience */
-    int layoutDefault(int rootPercent, bool crypto, bool updateTree=true);
     void addToCombo(QComboBox *combo, bool warnNasty = false) const;
     static QStringList split(const QString &devname);
     static QString join(const QString &drive, int partnum);
@@ -162,8 +168,12 @@ class PartMan : public QAbstractItemModel
     bool brave, gptoverride;
     void scanVirtualDevices(bool rescan);
     void resizeColumnsToFit();
-    bool formatLinuxPartition(const QString &devpath, const QString &format, bool chkBadBlocks, const QString &label);
-    bool prepareSubvolumes(DeviceItem *partit);
+    void preparePartitions();
+    void formatPartitions();
+    void prepareSubvolumes(DeviceItem *partit);
+    bool fixCryptoSetup();
+    bool makeFstab();
+    void mountPartitions();
     void treeItemChange();
     void treeSelChange();
     void treeMenu(const QPoint &);
@@ -176,8 +186,8 @@ class PartMan : public QAbstractItemModel
     void partMenuUnlock(DeviceItem *twit);
     void partMenuLock(DeviceItem *twit);
     void scanSubvolumes(DeviceItem *partit);
-    bool luksFormat(DeviceItem *partit, const QByteArray &password);
-    bool luksOpen(DeviceItem *partit, const QString &luksfs, const QByteArray &password);
+    void luksFormat(DeviceItem *partit, const QByteArray &password);
+    void luksOpen(DeviceItem *partit, const QString &luksfs, const QByteArray &password);
 public:
     enum TreeColumns {
         Device,
@@ -196,6 +206,7 @@ public:
     long long rootSpaceNeeded = 0;
     long long bootSpaceNeeded = 0;
     QMap<QString, DeviceItem *> mounts;
+    class AutoPart *autopart = nullptr;
     PartMan(MProcess &mproc, Ui::MeInstall &ui, const QSettings &appConf, const QCommandLineParser &appArgs);
     void scan(DeviceItem *drvstart = nullptr);
     bool manageConfig(MSettings &config, bool save);
@@ -204,18 +215,16 @@ public:
     DeviceItem *selectedDriveAuto();
     void clearAllUses();
     int countPrepSteps();
-    bool preparePartitions();
-    bool formatPartitions();
+    void prepStorage();
+    void installTabs();
     void loadKeyMaterial(const QString &keyfile);
-    bool fixCryptoSetup();
-    bool makeFstab(bool populateMediaMounts);
-    bool mountPartitions();
     void unmount();
     bool willFormat(const QString &point);
     QString getMountDev(const QString &point, const bool mapped=true);
     int swapCount();
     int isEncrypt(const QString &point);
     DeviceItem *findByPath(const QString &devpath) const;
+    DeviceItem *findHostDev(const QString &path) const;
     // Model View Controller
     QVariant data(const QModelIndex &index, int role) const override;
     bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole) override;
